# _*_ coding: utf-8 _*_
#
# hspylib v0.11.31
#
# Package: main.hspylib.addons.appman.templates
"""Package initialization."""

__all__ = [
  'tpl-main', 
  'tpl-main-qt', 
  'tpl-test_main', 
  'tpl-widget'
]
__version__ = '0.11.31'
